#!/usr/local/bin/perl

#
# $Header: /cvsroot/popick/popick.pl,v 1.5 2004/12/04 18:01:51 mina Exp $
#
# POPICK
# An interactive POP3-mailbox regex-based deleter
#
# Written by Mina Naguib <webmaster@topfx.com>
# http://www.topfx.com
#
# Copyright Mina Naguib - released under the GPL License
#

use strict;
use Getopt::Long;
use Net::POP3;
use Curses::UI;
use vars qw($HOSTNAME $USERNAME $PASSWORD);
use vars qw($CUI $WIN);
use vars qw($VERSION $POP3 %MESSAGEINFO);

$VERSION = '0.1';

initialize_ui();

get_variables();

pop3_connect();

pop3_download();

show_messages();

#
# This sub initializes the Curses::UI stuff into $CUI & co..
#
sub initialize_ui {
	$CUI = new Curses::UI(
		-clear_on_exit => 1,
		-color_support => 1,
	  )
	  || die "Failed to initialize new Curses::UI object: $! $@\n";
	$WIN = $CUI->add(
		"main_window", "Window",
		-title  => "POPICK v$VERSION",
		-border => 1,
		-ipad   => 1,
	);
	$CUI->set_binding(sub { exit(0) }, "\cC", "\cQ");
}

#
# This sub's job is to make sure the global variables $HOSTNAME and $USERNAME and $PASSWORD
# get populated via any means possible
# Otherwise prints out help to the user and exit()s
#
sub get_variables {
	local (*FH);
	my $filename;
	my $win;
	my ($hostname, $username, $password);
	my ($help,     $version);

	#
	# First, the config files
	#

	foreach $filename (grep { $_ } ("/etc/popick.conf", glob("~/.popick.conf"), glob("~/.popick"))) {

		if (open(FH, $filename)) {
			while (<FH>) {
				chomp;
				if (/^\s*hostname\s*=?\s*(.+?)\s*$/i) {
					$HOSTNAME = $1;
				}
				elsif (/^\s*username\s*=?\s*(.+?)\s*$/i) {
					$USERNAME = $1;
				}
				elsif (/^\s*password\s*=?\s*(.+?)\s*$/i) {
					$PASSWORD = $1;
				}
			}
		}
	}

	#
	# Next, environment variables
	#
	$HOSTNAME ||= $ENV{"POPICK_HOSTNAME"};
	$USERNAME ||= $ENV{"POPICK_USERNAME"};
	$PASSWORD ||= $ENV{"POPICK_PASSWORD"};

	#
	# Next, commandline switches
	#
	GetOptions(
		"hostname=s" => \$hostname,
		"username=s" => \$username,
		"password=s" => \$password,
		"help"       => \$help,
		"version"    => \$version,
	);

	if ($version) {
		$CUI->leave_curses();
		print "$0 version $VERSION\n";
		exit(0);
	}
	if ($help) {
		$CUI->leave_curses();
		print "\nHelp available by running:\n\n\tperldoc $0\n\n";
		exit(0);
	}

	$HOSTNAME ||= $hostname;
	$USERNAME ||= $username;
	$PASSWORD ||= $password;

	if (!$HOSTNAME || !$USERNAME || !$PASSWORD) {

		#
		# Finally, we beg for them!
		#
		$win = $WIN->add(
			"settings_window", "Window",
			-title    => "POP3 Settings",
			-border   => 1,
			-height   => 11,
			-width    => 30,
			-centered => 1,
		);

		$win->add(
			"hostname_label", "Label",
			-text  => "Hostname:",
			-x     => 1,
			-y     => 1,
			-width => 10,
		);
		$win->add(
			"hostname", "TextEntry",
			-text     => $HOSTNAME,
			-sbborder => 1,
			-x        => 11,
			-y        => 1,
			-tolower  => 1,
		)->focus();

		$win->add(
			"username_label", "Label",
			-text  => "Username:",
			-x     => 1,
			-y     => 3,
			-width => 10,
		);
		$win->add(
			"username", "TextEntry",
			-text     => $USERNAME,
			-sbborder => 1,
			-x        => 11,
			-y        => 3,
		);

		$win->add(
			"password_label", "Label",
			-text  => "Password:",
			-x     => 1,
			-y     => 5,
			-width => 10,
		);
		$win->add(
			"password", "PasswordEntry",
			-text     => $PASSWORD,
			-sbborder => 1,
			-x        => 11,
			-y        => 5,
		);

		$win->add(
			"buttons",
			"Buttonbox",
			-x       => 1,
			-y       => 7,
			-buttons => [
				{
					-label   => "< Connect >",
					-onpress => sub {
						my $self = shift;
						$HOSTNAME = $win->getobj("hostname")->get();
						$USERNAME = $win->getobj("username")->get();
						$PASSWORD = $win->getobj("password")->get();
						$self->parent->loose_focus();
					},
				},
				{
					-label   => "< Cancel >",
					-onpress => sub {
						exit;
					},
				},
			],
		);

		$win->modalfocus();

		$WIN->delete("settings_window");

	}

	#
	# Sanity
	#
	if (!$HOSTNAME) {
		exit_error("POP3 hostname must be specified");
	}
	elsif (!$USERNAME) {
		exit_error("POP3 username must be specified");
	}
	elsif (!$PASSWORD) {
		exit_error("POP3 password must be specified");
	}

}

#
# Connects to the POP3 server and sets $POP3
#
sub pop3_connect {

	$CUI->progress(
		-message => "Connecting to $HOSTNAME . . .",
		-max     => 3,
		-pos     => 1,
	);
	$POP3 = Net::POP3->new($HOSTNAME, Timeout => 30);
	if (!$POP3) {
		exit_error("Failed to connect to $HOSTNAME");
	}

	$CUI->setprogress(2, "Logging in . . .");
	if (!$POP3->login($USERNAME, $PASSWORD)) {
		exit_error("Failed to login: " . $POP3->message());
	}

	$CUI->setprogress(3, "Logged-in successfully!");

	$CUI->noprogress();
}

#
# Downloads the message headers into %MESSAGEINFO
#
sub pop3_download {
	my $messages = $POP3->list();
	my $headers;
	my $num;
	my $i;
	my $id;

	if (!$messages) {
		exit_error("Failed to retrieve messages list");
	}
	if (!keys %$messages) {
		exit_error("There are no messages on the server");
	}
	$num = scalar keys %$messages;

	$CUI->progress(
		-message => "Retrieving message headers . . .",
		-max     => $num,
		-pos     => 0,
	);

	$i = 0;
	foreach $id (keys %$messages) {
		$i++;
		$CUI->setprogress($i, "Retrieving message header [$i/$num] . . .");

		$headers = $POP3->top($id) || exit_error("Failed to retrieve headers for message ID $id");
		foreach (@$headers) {
			if (/^From:\s*(.+?)\s*$/i) {
				$MESSAGEINFO{$id}{from} = $1;
			}
			elsif (/^Subject:\s*(.+?)\s*$/i) {
				$MESSAGEINFO{$id}{subject} = $1;
			}
		}
	}

	$CUI->noprogress();
}

#
# This shows the stuff in %MESSAGEINFO and sets up the main loop etc
#
sub show_messages {
	my $win;
	my $list;
	my $regex;

	$win = $WIN->add(
		"regex_window", "Window",
		-title  => "Regular expression to match: ",
		-height => 5,
		-border => 1,
		-ipad   => 1,
	);

	$win->add(
		"regex_label", "Label",
		-text  => "Regex:",
		-x     => 0,
		-y     => 0,
		-width => 7,
	);
	$regex = $win->add(
		"regex", "TextEntry",
		-sbborder => 1,
		-x        => 7,
		-y        => 0,
		-bg       => "green",
		-fg       => "black",
	);

	$list = $WIN->add(
		"list", "Listbox",
		-title      => "Matching messages: ",
		-border     => 1,
		-htmltext   => 1,
		-vscrollbar => "right",
		-padtop     => 5,
		-ipad       => 1,
	);

	$list->set_routine("loose-focus",  sub { $regex->focus() });
	$regex->set_routine("loose-focus", sub { $list->focus() });
	$regex->onChange(\&_refresh_list);
	$CUI->set_binding(
		sub {

			#
			# This sub gets called when it's time to delete matching messages
			#
			my $regex = $WIN->getobj("regex_window")->getobj("regex")->get();
			my $id;
			my @matchedids;

			#
			# Sometimes the regex is bad
			#
			eval { /$regex/ };
			if ($@) {
				$CUI->status("Bad regex! Deletion not possible");
				sleep(3);
				$CUI->nostatus();
				return;
			}

			foreach $id (keys %MESSAGEINFO) {
				if ($MESSAGEINFO{$id}{from} =~ /$regex/i || $MESSAGEINFO{$id}{subject} =~ /$regex/i) {
					push(@matchedids, $id);
				}
			}

			if (!@matchedids) {
				$CUI->dialog("There is nothing to delete!");
			}
			else {
				my $yes = $CUI->dialog(
					-message => ("Are you sure you wish to delete the " . scalar(@matchedids) . " shown message(s) ?"),
					-buttons => [
						{
							-label => "< Yes >",
							-value => 1,
						},
						{
							-label => "< No >",
							-value => 0,
						},
					],
					-title => "Confirm deletion",
					-bg    => "red",
				);
				if ($yes) {

					$CUI->progress(
						-message => "Deleting messages . . .",
						-max     => scalar @matchedids,
						-pos     => 0,
					);

					my $i = 0;
					foreach $id (@matchedids) {
						$i++;
						$CUI->setprogress($i, "Deleting message $i id $id");
						$POP3->delete($id);
						delete $MESSAGEINFO{$id};
					}

					$CUI->noprogress();

					_refresh_list();
				}
			}

		},
		"\cX",
	);

	_refresh_list();

	$regex->focus();

	$CUI->mainloop();

}

#
# Takes a string
# Displays it
# then exit()s
#
sub exit_error {
	my $message = shift;
	$CUI->error($message);
	exit(0);
}

#
# Refreshes the list
#
sub _refresh_list {
	my $list     = $WIN->getobj("list");
	my $regexbox = $WIN->getobj("regex_window")->getobj("regex");
	my $regex    = $regexbox->get();
	my %matchinginfo;
	my $id;

	#
	# Sometimes the regex is bad
	#
	eval { /$regex/ };
	if ($@) {
		$regexbox->set_color_bg("red");
		$regexbox->set_color_fg("white");
		return;
	}
	else {
		$regexbox->set_color_bg("green");
		$regexbox->set_color_fg("black");
	}

	foreach $id (keys %MESSAGEINFO) {
		if ($MESSAGEINFO{$id}{from} =~ /$regex/i || $MESSAGEINFO{$id}{subject} =~ /$regex/i) {
			$matchinginfo{$id}{from}    = $MESSAGEINFO{$id}{from};
			$matchinginfo{$id}{subject} = $MESSAGEINFO{$id}{subject};
		}
	}
	$list->values([ keys %matchinginfo ]);
	$list->labels({ map { ($_, sprintf("<bold>%-32s</bold>: %s", substr($matchinginfo{$_}{from}, 0, 32), $matchinginfo{$_}{subject})) } keys %matchinginfo });
	$list->draw();

	$regexbox->focus();
}

=head1 NAME

popick - An interactive, regex-based POP3 mailbox deleter

=head1 SYNOPSIS

popick [--hostname hostname] [--username username] [--password password] | --help | --version

=head1 DESCRIPTION

popick is a simple POP3 mailbox deleter.  It allows you to interactively specify regular expressions to match the message headers, and delete matching messages.

=head1 OPTIONS

=over 4

=item hostname

	Specifies the POP3 mail server hostname to connect to

=item username

	Specifies the POP3 username to login to the server with

=item password

	Specifies the POP3 password to login to the server with

=item help

	This help

=item version

	Show program version and exit

=back

=head1 SPECIFYING OPTIONS

Options can be specified in all of the following ways.  When popick runs, it will scan and cumulatively use options from the following:

=over 4

=item Config file /etc/popick.conf

=item Config file ~/.popick.conf

=item Config file ~/.popick

=item Environment variables

=item Command-line switches

=item Manual prompting

=back

Any configuration variable specified via any of the above method will supercede the same variable set via an earlier method.

=head1 CONFIG FILES

The above-mentioned config files have a simple "key=value" format.  Everything else is ignored.

=head1 ENVIRONMENT VARIABLES

Environment variables must be all in uppercase and superceded with the prefix "POPICK_".  For example to specify the hostname via an environment variable:

	export POPICK_HOSTNAME=mail.isp.com

=head1 NAVIGATION

=over 4

=item TAB

	Moves between available fields

=item CTRL+Q, CTRL+C

	Exit

=item CTRL+X

	Delete matched messages shown in the messages list

=back

=head1 DEPENDENCIES

Requires the following perl modules to be installed:

=over 4

=item Getopt::Long

=item Net::POP3

=item Curses::UI

=back

=head1 COPYRIGHT

Copyright (C) 2004 Mina Naguib <webmaster@topfx.com>
http://www.topfx.com
Released under the GPL LICENSE version 2.0

=cut

END {
	if ($POP3) {
		$POP3->quit();
	}
}

#
# Leave me alone:
#
1;

